#include "stdafx.h"
#include "NMImgButton.h"


CNMImgButton::CNMImgButton()
{	
	m_bDisable = FALSE;
	m_bHover = FALSE;
	m_crText = RGB(0,0,0);

	// tracking
	m_bTracking = FALSE;

	// text
	m_strBmpText = _T("");
	m_bUseHandCursor = FALSE;
    m_bUISelected = FALSE;
    m_nMask = 0;
    m_rectButton.SetRectEmpty();
    m_rectButtonPos.SetRectEmpty();
}

CNMImgButton::~CNMImgButton()
{
}

BEGIN_MESSAGE_MAP(CNMImgButton, CButton)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_MESSAGE(WM_MOUSEHOVER,	OnMouseHover)
	ON_MESSAGE(WM_MOUSELEAVE,	OnMouseLeave)
	ON_WM_ERASEBKGND()
	ON_WM_SETCURSOR()
END_MESSAGE_MAP()

void CNMImgButton::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	CDC* pDC = CDC::FromHandle(lpDrawItemStruct->hDC);

	if (m_bDisable == TRUE)
		m_bmpImage.Draw(pDC, BUTTON_DISABLE, 0, 0, m_nMask, m_strBmpText);
	else
	{
		if (lpDrawItemStruct->itemState & ODS_SELECTED)
			m_bmpImage.Draw(pDC, BUTTON_CLICK, 0, 0, m_nMask, m_strBmpText);
		else if (m_bHover)
			m_bmpImage.Draw(pDC, BUTTON_HOVER, 0, 0, m_nMask, m_strBmpText);
		else
			m_bmpImage.Draw(pDC, BUTTON_ENABLE, 0, 0, m_nMask, m_strBmpText);
	}

	//ReleaseDC(pDC); //don't call ReleaseDC for FromHandle

	m_bHover = FALSE;
}

void CNMImgButton::SetBtnBitmapFile(CString strFileName,UINT nMask)
{
 	m_bmpImage.LoadUnitedBitmap(strFileName);
	m_nMask = nMask;

	m_rectButton.left	= 0;
	m_rectButton.top	= 0;
	m_rectButton.right	= m_bmpImage.GetSliceWidth();
	m_rectButton.bottom	= m_bmpImage.GetHeight();

	CWnd* pWnd = this->GetParent();
	if (pWnd == NULL)
		return;

	GetWindowRect(&m_rectButtonPos);
	pWnd->ScreenToClient(m_rectButtonPos);
	m_rectButtonPos.right = m_rectButtonPos.left + m_bmpImage.GetSliceWidth();
	m_rectButtonPos.bottom = m_rectButtonPos.top + m_bmpImage.GetHeight();
	MoveWindow(m_rectButtonPos);
}

int	CNMImgButton::GetImgWidth()
{
	return m_bmpImage.GetSliceWidth();
}

int	CNMImgButton::GetImgHeight()
{
	return m_bmpImage.GetHeight();
}

void CNMImgButton::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (!m_bTracking)
	{
		m_bTracking = TRUE;

		TRACKMOUSEEVENT tme;
		tme.cbSize = sizeof(TRACKMOUSEEVENT);
		tme.dwFlags = TME_HOVER | TME_LEAVE;
		tme.dwHoverTime = 1;
		tme.hwndTrack = GetSafeHwnd();
		TrackMouseEvent(&tme);
	}

	CButton::OnMouseMove(nFlags, point);
}


LRESULT CNMImgButton::OnMouseHover(WPARAM wparam, LPARAM lparam)
{
	m_bHover = TRUE;
	Invalidate(FALSE);

	return 0L;
}

LRESULT CNMImgButton::OnMouseLeave(WPARAM wparam, LPARAM lparam)
{
	m_bTracking = FALSE;
	m_bHover = FALSE;
	Invalidate(FALSE);

	return 0L;
}

void CNMImgButton::SetButtonEnable()
{
	m_bDisable = FALSE;
	Invalidate(FALSE);
}

void CNMImgButton::SetButtonDisable()
{
	m_bDisable = TRUE;
	Invalidate(FALSE);
}

void CNMImgButton::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (!m_bDisable)
		CButton::OnLButtonDown(nFlags, point);
	Invalidate(FALSE);
}

BOOL CNMImgButton::PreTranslateMessage(MSG* pMsg) 
{
	if (m_ToolTip.GetSafeHwnd() != NULL)
		m_ToolTip.RelayEvent(pMsg);

	return CButton::PreTranslateMessage(pMsg);
}

void CNMImgButton::SetToolTipText(CString strText)
{
	if (m_ToolTip.GetSafeHwnd() == NULL)
	{
		m_ToolTip.Create(this);
		m_ToolTip.Activate(TRUE);
	}
	m_ToolTip.AddTool(this, (LPCTSTR)strText);
}

BOOL CNMImgButton::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void CNMImgButton::SetSelected(bool bSelected)
{
	m_bUISelected = bSelected;
	Invalidate(FALSE);
}

BOOL CNMImgButton::EnableWindow(BOOL bEnable)
{
	if (bEnable)
	{
		SetButtonEnable();
		m_bmpImage.SetTxtColor(m_crText);
	}
	else
	{
		SetButtonDisable();
		m_bmpImage.SetTxtColor(RGB(100,100,100));
	}

	return CButton::EnableWindow(bEnable);
}

void CNMImgButton::SetBmpFontColor( COLORREF col )
{
	m_crText = col;
	m_bmpImage.SetTxtColor(m_crText);
}

BOOL CNMImgButton::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	if (m_bUseHandCursor)
	{
		::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_HAND));
		return TRUE;
	}

	return CButton::OnSetCursor(pWnd, nHitTest, message);
}

void CNMImgButton::EnableHandCursor(BOOL bUseHand)
{
	m_bUseHandCursor = bUseHand;
}
